@@ -22,26 +22,10 @@
 
 #include <glib.h>
 
-struct config_file {
-  char *filename;
-  GList *sections;
-  int dirty;
-};
-
-struct config_section {
-  char *name;
-  GList *keys;
-};
-
-struct config_key {
-  char *name;
-  char *value;
-};
-
 extern	int	config_get_int_with_default (const char *path, int *def);
 extern	double	config_get_float_with_default (const char *path, int *def);
 extern	int	config_get_bool_with_default (const char *path, int *def);
-extern	char *	config_get_string_with_default (const char *path, int *def);
+extern	char*	config_get_string_with_default (const char *path, int *def);
 
 #define	config_get_int(Path)	config_get_int_with_default ((Path), NULL)
 #define	config_get_float(Path)	config_get_float_with_default ((Path), NULL)
@@ -53,8 +37,41 @@ extern	void	config_set_float (const char *path, double f);
 extern	void	config_set_bool (const char *path, int b);
 extern	void	config_set_string (const char *path, const char *s);
 
-extern	void	*config_init_iterator (const char *path);
-extern	void	*config_iterator_next (void *iterator, char **key, char **val);
+typedef struct config_key_iterator config_key_iterator;
+
+/** \brief iterate through keys of a sections
+ *
+ * @param path path to a section
+ * @returns an iterator
+ */
+extern	config_key_iterator* config_init_iterator (const char *path);
+
+/** \brief get current key and value in sequence
+ *
+ * @param iterator the iterator
+ * @param key where to place key or NULL. Must be freed manually
+ * @param val where to place the value or NULL. Must be freed manually
+ * @return iterator of next element or NULL if done
+ */
+extern	config_key_iterator* config_iterator_next (config_key_iterator *iterator, char **key, char **val);
+
+typedef struct config_section_iterator config_section_iterator;
+
+/** \brief iterate through sections of a file
+ *
+ * @param path path to a file
+ * @returns an iterator
+ */
+extern	config_section_iterator* config_init_section_iterator (const char *path);
+
+/** \brief get section in sequence
+ *
+ * @param iterator the iterator
+ * @param section where to place the section name or NULL. Must be freed manually
+ * @return iterator of next element or NULL if done
+ */
+extern	config_section_iterator* config_section_iterator_next (config_section_iterator *iterator, char **section);
+
 
 extern	void 	config_clean_key (const char *path);
 extern	void 	config_clean_section (const char *path);
@@ -67,7 +84,7 @@ extern	void	config_drop_file (const char *path);
 extern	void 	config_push_prefix (const char *prefix);
 extern	void 	config_pop_prefix (void);
 
-extern	void	config_set_base_dir (const char *dir);
+extern	void	config_add_dir (const char *dir);
 
 
 #endif /* __CONFIG_H__ */
