@@ -20,10 +20,26 @@
 #define _DEBUG_H_
 
 #define DEFAULT_DEBUG_LEVEL 0
-void debug(int, char *, ...);
+
+#define debug(level,fmt,rest...) \
+  if( level <= get_debug_level() ) \
+    debug_int(__FILE__,__LINE__,__FUNCTION__,level,fmt,##rest)
+
+#define xqf_informal(fmt,rest...) \
+  debug_int(__FILE__,__LINE__,__FUNCTION__,-1,fmt,##rest)
+
+#define xqf_warning(fmt,rest...) \
+  debug_int(__FILE__,__LINE__,__FUNCTION__,-2,fmt,##rest)
+
+#define xqf_error(fmt,rest...) \
+  debug_int(__FILE__,__LINE__,__FUNCTION__,-3,fmt,##rest)
+
+void debug_int(const char* file, int line, const char* function, int level, const char* fmt, ...);
 void debug_cmd(int, char *[], char *, ...);
 void set_debug_level (int);
-int get_debug_level (void);
-int debug_level;
+inline int get_debug_level (void);
+
+int debug_increase_indent(void);
+int debug_decrease_indent(void);
 
 #endif
