@@ -56,6 +56,7 @@ extern void qw_cmd_or_dir_changed(struct game* g);
 extern void q2_cmd_or_dir_changed(struct game* g);
 extern void ut2004_cmd_or_dir_changed(struct game* g);
 extern void doom3_cmd_or_dir_changed(struct game* g);
+extern void tremulous_cmd_or_dir_changed(struct game* g);
 
 static struct player *poqs_parse_player(char *tokens[], int num, struct server *s);
 static struct player *qw_parse_player(char *tokens[], int num, struct server *s);
@@ -161,6 +162,8 @@ static struct quake_private cod_private;
 static struct quake_private jk3_private;
 static struct quake_private doom3_private;
 static struct quake_private quake4_private;
+static struct quake_private warsow_private;
+static struct quake_private tremulous_private;
 
 #include "games.c"
 
@@ -188,7 +191,7 @@ static struct gsname2type_s gsname2type[] =
 
 void init_games()
 {
-  unsigned i;
+  unsigned i, j;
 
   debug(3,"initializing games");
 
@@ -197,6 +200,14 @@ void init_games()
     g_datalist_init(&games[i].games_data);
   }
 
+  for (i = 0; i < GAMES_TOTAL; i++)
+  {
+    for (j = 0; games[i].attributes && games[i].attributes[j]; j += 2)
+    {
+      game_set_attribute_const(i,games[i].attributes[j], games[i].attributes[j+1]);
+    }
+  }
+
   game_set_attribute(SFS_SERVER,"game_notes",strdup(_
    				   ("Note:  Soldier of Fortune will not connect to a server correctly\n"\
     				    "without creating a startup script for the game.  Please see the\n"\
@@ -205,7 +216,7 @@ void init_games()
   				   ("Note:  Unreal Tournament will not launch correctly without\n"\
     				    "modifications to the game's startup script.  Please see the\n"\
 			  	    "XQF documentation for more information.")));
-  game_set_attribute(HL_SERVER,"game_notes",strdup(_
+  game_set_attribute(HL_SERVER_OLD,"game_notes",strdup(_
   				   ("Sample Command Line:  wine hl.exe -- hl.exe -console")));
 
   game_set_attribute(SAS_SERVER,"game_notes",strdup(_
@@ -238,6 +249,14 @@ const char* game_set_attribute(enum server_type type, const char* attr, char* va
   return value;
 }
 
+// set game specific key/value pair, value is _not_ copied and must not be
+// freed manually
+const char* game_set_attribute_const(enum server_type type, const char* attr, const char* value)
+{
+  g_datalist_set_data_full(&games[type].games_data,attr,(void*)value,NULL);
+  return value;
+}
+
 enum server_type id2type (const char *id) {
   int i;
 
@@ -307,7 +326,7 @@ GtkWidget *game_pixmap_with_label (enum server_type type) {
 }
 
 
-static void q3_unescape (char *dst, char *src) {
+static void q3_unescape (char *dst, const char *src) {
 
   while (*src) {
     if (src[0] != '^' || src[1] == '\0' || src[1] < '0' || src[1] > '9')
@@ -453,7 +472,7 @@ static struct player *descent3_parse_player (char *token[], int n, struct server
 
 static struct player *q3_parse_player (char *token[], int n, struct server *s) {
   struct player *player = NULL;
-  char* clan = NULL;
+  const char* clan = NULL;
   unsigned clanlen = 0;
 
   if (n < 3)
@@ -465,6 +484,15 @@ static struct player *q3_parse_player (char *token[], int n, struct server *s) {
     clanlen = strlen(clan)+1;
   }
 
+  if(s->type == WARSOW_SERVER && clan)
+  {
+    static const char* colors[] = { "spectator", "", "red", "blue", "green", "yellow" };
+    unsigned i = atoi(clan);
+    if(i > 5) i = 1;
+    clan = colors[i];
+    clanlen = 0;
+  }
+
   player = g_malloc0 (sizeof (struct player) + strlen (token[0]) + 1 + clanlen);
   player->time  = -1;
   player->frags = strtosh (token[1]);
@@ -478,8 +506,15 @@ static struct player *q3_parse_player (char *token[], int n, struct server *s) {
   // show clan name in model column
   if(clan)
   {
-    player->model = (char *) player + sizeof (struct player) + strlen(player->name) + 1; 
-    strcpy(player->model, clan);
+    if(s->type == WARSOW_SERVER)
+    {
+      player->model = (char*)clan;
+    }
+    else
+    {
+      player->model = (char *) player + sizeof (struct player) + strlen(player->name) + 1; 
+      q3_unescape(player->model, clan);
+    }
   }
 
   return player;
@@ -566,7 +601,7 @@ static void quake_parse_server (char *token[], int n, struct server *server) {
       return;
 
   if (*(token[2])) {		/* if name is not empty */
-    if (server->type != Q3_SERVER) {
+    if ((games[server->type].flags & GAME_Q3COLORS) == 0) {
       server->name = g_strdup (token[2]);
     }
     else {
@@ -1134,6 +1169,18 @@ static char *wolfet_gametypes[MAX_WOLFET_TYPES] = {
   NULL			// 6+ ???
 };
 
+#define MAX_WOLFET_TCETEST_TYPES 8
+static char *wolfet_tcetest_gametypes[MAX_WOLFET_TCETEST_TYPES] = {
+  NULL,			// 0 - Unknown
+  NULL,			// 1 - Unknown
+  NULL,			// 2 - Unknown
+  NULL,			// 3 - Unknown
+  NULL,			// 4 - Unknown
+  "Demolition",		// 5 - Demolition
+  NULL,			// 6 - Unknown
+  "Bodycount"			// 7 - Bodycount
+};
+
 #define MAX_JK3_TYPES 9
 static char *jk3_gametypes[MAX_JK3_TYPES] = {
   "FFA",		// 0 - Free For All
@@ -1352,6 +1399,11 @@ struct q3a_gametype_s wolfet_gametype_map[] =
     "shrubet",
     wolfet_gametypes,
     MAX_WOLFET_TYPES
+  },
+  {
+    "tcetest",
+    wolfet_tcetest_gametypes,
+    MAX_WOLFET_TCETEST_TYPES
   }
 };
 
@@ -1449,6 +1501,10 @@ static void q3_analyze_serverinfo (struct server *s) {
       {
 	s->type=SOF2S_SERVER;
       }
+      else if(!strncmp(info_ptr[1],"tremulous",9))
+      {
+	s->type=TREMULOUS_SERVER;
+      }
 
       break;
     }
@@ -1492,6 +1548,10 @@ static void q3_analyze_serverinfo (struct server *s) {
 	strcmp (*info_ptr, "g_gametypestring") == 0) {
 	s->gametype = info_ptr[1];
     }
+    else if (s->type == WARSOW_SERVER &&
+	strcmp (*info_ptr, "g_gametype_name") == 0) {
+	s->gametype = info_ptr[1];
+    }
     else if (strcmp (*info_ptr, "g_needpass") == 0) {
       n = strtol (info_ptr[1], NULL, 10);
       if ((n & 1) != 0)
@@ -2251,15 +2311,11 @@ static int q3_exec (const struct condef *con, int forkit) {
   
   char *real_game_dir = NULL;
 
-  int vmfix = 0;
   int setfs_game = 0;
   int set_punkbuster = 0;
+  int enable_console = 0;
   int pass_memory_options = 0;
   
-  int vm_game_set = 0;
-  int vm_cgame_set = 0;
-  int vm_ui_set = 0;
-
   struct quake_private* pd = NULL;
   
   if(!con) return -1;
@@ -2270,9 +2326,9 @@ static int q3_exec (const struct condef *con, int forkit) {
 
   pd = (struct quake_private*)games[con->s->type].pd;
 
-  vmfix               = str2bool(game_get_attribute(g->type,"vmfix"));
   setfs_game          = str2bool(game_get_attribute(g->type,"setfs_game"));
   set_punkbuster      = str2bool(game_get_attribute(g->type,"set_punkbuster"));
+  enable_console      = str2bool(game_get_attribute(g->type,"enable_console"));
   pass_memory_options = str2bool(game_get_attribute(g->type,"pass_memory_options"));
 
   cmdtokens = g_strsplit(g->cmd," ",0);
@@ -2339,7 +2395,11 @@ static int q3_exec (const struct condef *con, int forkit) {
 
   if (con->rcon_password)
   {
-    argv[argi++] = "+rconpassword";
+    const char* pwvarname = NULL;
+    if((pwvarname = game_get_attribute(con->s->type, "+rconpassword")))
+      argv[argi++] = (char*)pwvarname;
+    else
+      argv[argi++] = "+rconpassword";
     argv[argi++] = con->rcon_password;
   }
 
@@ -2369,31 +2429,34 @@ static int q3_exec (const struct condef *con, int forkit) {
     argv[argi++] = con->custom_cfg;
   }
 
-// useful for wolf too, ef doesn't have it
-//  if(g->type == Q3_SERVER)
+  /* The 1.32 release of Q3A needs +set cl_punkbuster 1 on the command line. */
+  punkbuster = find_server_setting_for_key ("sv_punkbuster", con->s->info);
+  if( punkbuster != NULL && strcmp( punkbuster, "1" ) == 0 )
   {
-    /* The 1.32 release of Q3A needs +set cl_punkbuster 1 on the command line. */
-    punkbuster = find_server_setting_for_key ("sv_punkbuster", con->s->info);
-    if( punkbuster != NULL && strcmp( punkbuster, "1" ) == 0 )
+    if( set_punkbuster )
     {
-      if( set_punkbuster )
-      {
-	argv[argi++] = "+set";
-	argv[argi++] = "cl_punkbuster";
-	argv[argi++] = "1";
-      }
-      else
-      {
-	char* option = g_strdup_printf("/" CONFIG_FILE "/Game: %s/punkbuster dialog shown",type2id(g->type));
-	debug( 1, "Got %s for punkbuster\n", punkbuster );
-	if(!config_get_bool (option))
-	{
-	  dialog_ok (NULL, _("The server has Punkbuster enabled but it is not going\nto be set on the command line.\nYou may have problems connecting.\nYou can fix this in the game preferences."));
-	  config_set_bool (option,TRUE);
-	}
-	g_free(option);
-      }
+      argv[argi++] = "+set";
+      argv[argi++] = "cl_punkbuster";
+      argv[argi++] = "1";
     }
+    else
+    {
+      char* option = g_strdup_printf("/" CONFIG_FILE "/Game: %s/punkbuster dialog shown",type2id(g->type));
+      debug( 1, "Got %s for punkbuster\n", punkbuster );
+      if(!config_get_bool (option))
+      {
+	dialog_ok (NULL, _("The server has Punkbuster enabled but it is not going\nto be set on the command line.\nYou may have problems connecting.\nYou can fix this in the game preferences."));
+	config_set_bool (option,TRUE);
+      }
+      g_free(option);
+    }
+  }
+
+  if(enable_console)
+  {
+    argv[argi++] = "+set";
+    argv[argi++] = "com_allowConsole";
+    argv[argi++] = "1";
   }
   
   /*
@@ -2457,38 +2520,9 @@ static int q3_exec (const struct condef *con, int forkit) {
   {
     argv[argi++] = additional_args[i];
 
-    // Check to see if vm_game, vm_cgame or vm_ui was set in custom args
-    // Used to prevent them from being re-set if vmfix is enabled below.
-    if (strcmp(additional_args[i], "vm_game") == 0)
-      vm_game_set = 1;
-    else if (strcmp(additional_args[i], "vm_cgame") == 0)
-      vm_cgame_set = 1;
-    else if (strcmp(additional_args[i], "vm_ui") == 0)
-      vm_ui_set = 1;
     i++;
   }
 
-  if(vmfix) {
-    if (!vm_game_set)
-    {
-      argv[argi++] = "+set";
-      argv[argi++] = "vm_game";
-      argv[argi++] = "2";
-    }
-    if (!vm_cgame_set)
-    {
-      argv[argi++] = "+set";
-      argv[argi++] = "vm_cgame";
-      argv[argi++] = "2";
-    }
-    if (!vm_ui_set)
-    {
-      argv[argi++] = "+set";
-      argv[argi++] = "vm_ui";
-      argv[argi++] = "2";
-    }
-  }    
-   
   argv[argi] = NULL;
 
   debug(1,"argument count %d", argi);
@@ -2516,11 +2550,6 @@ static int hl_exec (const struct condef *con, int forkit) {
   while ((argv[argi] = strtok (NULL, delim)) != NULL)
     argi++;
 
-  if(con->s->type == HL2_SERVER) // XXX
-  {
-    argv[argi++] = "-steam";
-  }
-
   if (con->gamedir) {
     argv[argi++] = "-game";
     argv[argi++] = con->gamedir;
@@ -3286,6 +3315,18 @@ static void quake_save_info (FILE *f, struct server *s) {
 		 p->model?p->model:"0");
 	break;
 
+      case HL_SERVER:
+      case HL_SERVER_OLD:
+      case HL2_SERVER:
+	fprintf (f, 
+		 "%s" QSTAT_DELIM_STR 
+		 "%d" QSTAT_DELIM_STR 
+		 "%d\n",
+		 (p->name)? p->name : "",
+		 p->frags,
+		 p->time);
+	break;
+
       default:
 	fprintf (f, 
 		 "%s" QSTAT_DELIM_STR 
@@ -3293,7 +3334,7 @@ static void quake_save_info (FILE *f, struct server *s) {
 		 "%d\n",
 		 (p->name)? p->name : "",
 		 p->frags,
-		 (s->type == HL_SERVER)? p->time : p->ping);
+		 p->ping);
 	break;
 
       } /* switch (s->type) */
