@@ -42,6 +42,8 @@ enum {
   GAME_QUAKE3_MASTERPROTOCOL	= 0x0400, // master server protocol version is in games_data["masterprotocol"]
   GAME_LAUNCH_HOSTPORT		= 0x0800, // use hostport rule as port when launching
   GAME_MASTER_CDKEY		= 0x1000, // master server requires CD key
+  GAME_Q3COLORS			= 0x2000, // Q3 color codes
+  GAME_MASTER_STEAM		= 0x4000, // server side filter
 };
 
 struct game {
@@ -97,6 +99,9 @@ struct game {
   /** called when either cmd or dir have changed. Also on xqf startup */
   void (*cmd_or_dir_changed)(struct game* g);
 
+  /** load preferences from config */
+  void (*prefs_load)(struct game* g);
+
   /** sync preference dialog to config */
   void (*update_prefs)(struct game* g);
 
@@ -112,6 +117,9 @@ struct game {
   /** NULL terminated list of commands to search in $PATH for suggestion */
   char** command;
 
+  /** NULL terminated list attributes and keys. used for filling games_data  */
+  char** attributes;
+
   char *game_cfg;
   GData *games_data;
   GSList *custom_args;
@@ -127,9 +135,12 @@ extern	GtkWidget *game_pixmap_with_label (enum server_type type);
 
 // retreive game specific value that belongs to key, do not free return value!
 const char* game_get_attribute(enum server_type type, const char* key);
-// set game specific key/value pair, value is _not_ copied
+// set game specific key/value pair, value is _not_ copied but will be freed internally
 const char* game_set_attribute(enum server_type type, const char* key, char* value);
 
+// set game specific key/value pair, value is _not_ copied and will not be freed internally
+const char* game_set_attribute_const(enum server_type type, const char* attr, const char* value);
+
 void init_games(void);
 void games_done(void);
 
