@@ -53,6 +53,7 @@ typedef enum
     TAG_identify_cpu,
     TAG_identify_os,
     TAG_cmd_or_dir_changed,
+    TAG_prefs_load,
     TAG_update_prefs,
     TAG_default_home,
     TAG_pd,
@@ -62,7 +63,8 @@ typedef enum
     TAG_data = TAG_start_multi,
     TAG_main_mod,
     TAG_command,
-    TAG_end_multi = TAG_command,
+    TAG_attributes,
+    TAG_end_multi = TAG_attributes,
 
     TAG_base,
 
@@ -127,12 +129,14 @@ static void tags_init()
     add_tag(TAG_identify_cpu,       Tag_do_inherit, Tag_type_literal, "identify_cpu");
     add_tag(TAG_identify_os,        Tag_do_inherit, Tag_type_literal, "identify_os");
     add_tag(TAG_cmd_or_dir_changed, Tag_do_inherit, Tag_type_literal, "cmd_or_dir_changed");
+    add_tag(TAG_prefs_load,         Tag_no_inherit, Tag_type_literal, "prefs_load");
     add_tag(TAG_update_prefs,       Tag_no_inherit, Tag_type_literal, "update_prefs");
     add_tag(TAG_default_home,       Tag_no_inherit, Tag_type_string,  "default_home");
     add_tag(TAG_pd,                 Tag_no_inherit, Tag_type_address, "pd");
     add_tag(TAG_data,               Tag_no_inherit, Tag_type_literal, "data");
     add_tag(TAG_main_mod,           Tag_no_inherit, Tag_type_string,  "main_mod");
     add_tag(TAG_command,            Tag_no_inherit, Tag_type_string,  "command");
+    add_tag(TAG_attributes,         Tag_no_inherit, Tag_type_string,  "attributes");
     add_tag(TAG_base,               Tag_no_inherit, Tag_type_literal, "base");
     add_tag(TAG_invalid,            Tag_no_inherit, Tag_type_literal, NULL);
 }
