@@ -1,5 +1,5 @@
 /* XQF - Quake server browser and launcher
- * Copyright (C) 1998 Roman Pozlevich <roma@botik.ru>
+ * Copyright (C) 1998-2000 Roman Pozlevich <roma@botik.ru>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@ extern	void client_detach_all (void);
 extern	int client_launch_exec (int forkit, char *dir, char *argv[], 
                                                             struct server *s);
 
-extern 	void client_launch (const struct condef *con, int forkit);
+extern 	int client_launch (const struct condef *con, int forkit);
 
 extern	struct condef *condef_new (struct server *s);
 extern	void condef_free (struct condef *con);
