@@ -1,5 +1,5 @@
 /* XQF - Quake server browser and launcher
- * Copyright (C) 1998 Roman Pozlevich <roma@botik.ru>
+ * Copyright (C) 1998-2000 Roman Pozlevich <roma@botik.ru>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,13 +25,14 @@
 
 enum keyword_type {
   KEYWORD_INT,
+  KEYWORD_BOOL,
   KEYWORD_STRING
 };
 
 struct keyword {
   char *name;
   enum keyword_type required;
-  gpointer target;
+  char *config;
 };
 
 
