@@ -86,7 +86,7 @@ static void rcon_print (char *fmt, ...) {
 
   if (fmt) {
     va_start (ap, fmt);
-    g_vsnprintf (buf, 2048, fmt, ap);
+    g_vsnprintf (buf, sizeof(buf), fmt, ap);
     va_end (ap);
 
     gtk_text_freeze (GTK_TEXT (rcon_text));
@@ -209,7 +209,7 @@ static int rcon_send(const char* cmd)
     buf = g_strdup_printf("\377\377\377\377rcon %s %s %s",rcon_challenge, rcon_password, cmd);
     bufsize = strlen(buf)+1;
   }
-  else if (rcon_servertype == DOOM3_SERVER)
+  else if (rcon_servertype == DOOM3_SERVER || rcon_servertype == Q4_SERVER)
   {
     const char prefix[] = "\377\377rcon";
     bufsize = sizeof(prefix) +strlen(rcon_password) +1 +strlen(cmd) +1;
@@ -315,6 +315,7 @@ static char* msg_terminate (char *msg, int size)
 static char* rcon_receive()
 {
   char *msg = "\n";
+  ssize_t t;
   ssize_t size;
 
   if (!packet)
@@ -335,9 +336,30 @@ static char* rcon_receive()
       // "\377\377\377\377<some character>"
       msg = packet + 4 + 1;
       size = size - 4 - 1;
+
+      for(t = 0; t < size && msg[t]; t++)      // filter QW KTPRO status list
+      {
+        if((unsigned char)msg[t] == 141)
+          msg[t] = '>';
+
+        msg[t]&=0x7f;
+
+        if(msg[t] < 32)
+        {
+          if(msg[t] >= 0x12 && msg[t] <= 0x12 + 9)
+            msg[t]+='0' - 0x12;    // yellow numbers
+          else if(msg[t] == 16)
+            msg[t] = '[';
+          else if(msg[t] == 17)
+            msg[t] = ']';
+          else if(msg[t] != '\n' && msg[t] != '\r')
+            msg[t] = '.';        // unprintable
+        }
+      }
       break;
 
     case DOOM3_SERVER:
+    case Q4_SERVER:
       // "\377\377print\0????\0"
       if(size > 2+6+4+1)
       {
@@ -693,7 +715,7 @@ int main(int argc, char* argv[])
   if( argc-argpos<2 || !strcmp(argv[argpos],"--help"))
   {
       printf(_("Usage: %s [server type] <ip> <port>\n"),argv[0]);
-      printf(_("  server type is either --qws, --hws or --hls.\n"));
+      printf(_("  server type is either --qws, --hws, --hls or --dm3s.\n"));
       printf(_("  If no server type is specified, Q3 style rcon is assumed.\n"));
       return 1;
   }
