@@ -524,6 +524,7 @@ static void read_server_info (const char *filename) {
   char *buf;
   char *pos;
   GSList *strings;
+  int line = 1;
 
   debug(3,"read_server_info(%s)",filename);
 
@@ -541,7 +542,7 @@ static void read_server_info (const char *filename) {
 
   while (1) {
     if (pos - buf > 1024*16 - 16) {
-      fprintf (stderr, "server record is too large\n");
+      xqf_error("server record is too large in %s, line %d\n", realname, line);
       g_slist_free (strings);
       break;
     }
@@ -560,6 +561,7 @@ static void read_server_info (const char *filename) {
     else {
 
       if (pos[0] == '\n') {		/* empty line */
+	++line;
 	parse_saved_server (strings);
 	g_slist_free (strings);
 	strings = NULL;
@@ -570,6 +572,7 @@ static void read_server_info (const char *filename) {
 
 	while (*pos) {
 	  if (*pos == '\n') {
+	    ++line;
 	    *pos = '\0';
 	    break;
 	  }
@@ -1072,6 +1075,8 @@ static char *builtin_masters_update_info[] = {
   "ADD HLS,-stm master://steam1.steampowered.com Steam 1",
   "ADD HLS,-stm master://steam2.steampowered.com Steam 2",
 
+  "ADD HLA2S master://steam1.steampowered.com Steam 1",
+
   "ADD A2S,-stma2s master://steam1.steampowered.com:27011 Steam 1",
   "ADD A2S,-stma2s master://steam2.steampowered.com:27011 Steam 2",
   "ADD A2S,-stma2s master://69.28.151.178:27011 Valve 1",
@@ -1123,6 +1128,8 @@ static char *builtin_masters_update_info[] = {
   "DELETE MHS http://tourneys.multiplay.co.uk/public/servers.pl?opt=ListGamespy&event=Online&type=MOHAA&hostport=1 multiplay.co.uk - MOHAA", // does no longer work
   "DELETE EFS http://tourneys.multiplay.co.uk/public/servers.pl?opt=ListGamespy&event=Online&type=EF&hostport=1 multiplay.co.uk - Elite Force", // does no longer work
 
+  "ADD AMS http://simplembs.armygame.com/sparse.txt armygame.com",
+
   // version 2.0 not listed here
   "DELETE AMS http://tourneys.multiplay.co.uk/public/servers.pl?opt=ListGamespy&event=Online&type=AA&hostport=1 multiplay.co.uk - Army Ops", // does no longer work
   "DELETE SOF2S http://tourneys.multiplay.co.uk/public/servers.pl?opt=ListGamespy&event=Online&type=SOF2&hostport=1 multiplay.co.uk - SOF2", // does no longer work
@@ -1146,6 +1153,11 @@ static char *builtin_masters_update_info[] = {
   "ADD NEXUIZS master://ghdigital.com ghdigital.com",
   "ADD NEXUIZS master://dpmaster.deathmask.net dpmaster.deathmask.net",
 
+  "ADD WARSOWS master://ghdigital.com ghdigital.com",
+  "ADD WARSOWS master://dpmaster.deathmask.net dpmaster.deathmask.net",
+
+  "ADD TREMULOUS master://master.tremulous.net:30710 tremulous.net",
+
   "ADD QS lan://255.255.255.255 LAN",
   "ADD QWS lan://255.255.255.255 LAN",
   "ADD Q2S lan://255.255.255.255 LAN",
@@ -1176,6 +1188,7 @@ static char *builtin_gslist_masters_update_info[] = {
   "ADD QWS gslist://master.gamespy.com;gsmtype=quakeworld Gslist",
   "ADD Q2S gslist://master.gamespy.com;gsmtype=quake2 Gslist",
   "ADD Q3S gslist://master.gamespy.com;gsmtype=quake3 Gslist",
+  "ADD Q4S gslist://master.gamespy.com;gsmtype=quake4 Gslist",
   "ADD WOS gslist://master.gamespy.com;gsmtype=rtcw Gslist",
   "ADD WOETS gslist://master.gamespy.com;gsmtype=rtcwet Gslist",
   "ADD DM3S gslist://master.gamespy.com;gsmtype=doom3 Gslist",
@@ -1187,6 +1200,7 @@ static char *builtin_gslist_masters_update_info[] = {
   "ADD POSTAL2 gslist://master.gamespy.com;portadjust=-1;gsmtype=postal2d Gslist (Demo)",
   "ADD AMS gslist://master.gamespy.com;portadjust=-1;gsmtype=armygame Gslist",
   "ADD D3G gslist://master.gamespy.com;gsmtype=descent3 Gslist",
+  "ADD SOF2S gslist://master.gamespy.com;gsmtype=sof2 Gslist",
 //  "ADD GPS gslist://master.gamespy.com;gsmtype=mohaa Gslist", // no fixed port offset
 //  "ADD SMS gslist://master.gamespy.com;portadjust=-1;gsmtype=serioussam Gslist", // not compatible with linux version
 //  "ADD SMSSE gslist://master.gamespy.com;portadjust=-1;gsmtype=serioussamse Gslist", // not compatible with linux version
@@ -1666,3 +1680,27 @@ gboolean have_gslist_installed()
   g_free(gslist);
   return (gslist != NULL);
 }
+
+void master_remove_server(struct master* m, struct server* s)
+{
+  if(!m || !s)
+    return;
+
+  m->servers = server_list_remove(m->servers, s);
+}
+
+void server_remove_from_all(struct server *s)
+{
+  GSList *list;
+  struct master *m;
+
+  for (list = all_masters; list; list = list->next)
+  {
+    m = (struct master *) list->data;
+
+    if (m == favorites)
+      continue;
+
+    m->servers = server_list_remove(m->servers, s);
+  }
+}
