@@ -28,7 +28,7 @@
 
 
 #ifndef HTTP_HELPER
-# define HTTP_HELPER		"wget -q -e robots=off -O -"
+# define HTTP_HELPER		"wget -t 1 -T 20 -q -e robots=off --user-agent=XQF/" VERSION " -O -"
 #endif
 
 
@@ -40,6 +40,10 @@
 #define BUFFER_MAXSIZE	(128*1024)
 #define BUFFER_TRESHOLD	64
 
+// EVENT_* used to decide what sound to play
+#define EVENT_REFRESH_SELECTED 1
+#define EVENT_REFRESH          2
+#define EVENT_UPDATE           3
 
 struct stat_job;
 
@@ -64,6 +68,10 @@ struct stat_conn {
   struct stat_job *job;
 
   char *tmpfile;
+
+  // UGLY HACKS
+  gboolean first; // marker for savage check header
+  gboolean is_savage; // true if file is in savage format
 };
 
 struct delayed_refresh {
