@@ -16,10 +16,15 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#include "gnuconfig.h"
+
 #include <X11/Xlib.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
+#include "pixmaps.h"
+#include "loadpixmap.h"
+
 
 void iconify_window (GdkWindow *window) {
   Window xwindow;
@@ -27,8 +32,33 @@ void iconify_window (GdkWindow *window) {
   if (!window)
     return;
 
-  xwindow = ((GdkWindowPrivate*) window)->xwindow;
+
+
+  xwindow = GDK_WINDOW_XWINDOW(window);
+
+
   XIconifyWindow (GDK_DISPLAY (), xwindow, DefaultScreen (GDK_DISPLAY ()));
 }
 
+static const char* minimize_icon = "xqf_48x48.png";
 
+void window_set_icon (GtkWidget *win)
+{
+#ifdef USE_GTK2
+  GdkPixbuf* pixbuf;
+  pixbuf = load_pixmap_as_pixbuf(minimize_icon);
+  if(pixbuf)
+  {
+    gtk_window_set_icon (GTK_WINDOW (win), pixbuf);
+    gdk_pixbuf_unref (pixbuf);
+  }
+#else
+  static struct pixmap pix;
+
+  if(!load_pixmap_as_pixmap(win, minimize_icon, &pix))
+    return;
+
+  gdk_window_set_icon(win->window, NULL, pix.pix, pix.mask);
+  gdk_window_set_icon_name(win->window, "XQF");
+#endif
+}
